@@ -84,6 +84,30 @@ class PIDLockFile(LinkFileLock, object):
         super(PIDLockFile, self).break_lock()
         remove_existing_pidfile(self.path)
 
+
+class TimeoutPIDLockFile(PIDLockFile):
+    """ Lockfile with default timeout, implemented as a Unix PID file.
+
+        This uses the ``PIDLockFile`` implementation, with the
+        following changes:
+
+        * The `acquire_timeout` parameter to the initialiser will be
+          used as the default `timeout` parameter for the `acquire`
+          method.
+
+        """
+
+    def __init__(self, path, acquire_timeout=None, *args, **kwargs):
+        """ Set up the parameters of a DaemonRunnerLock. """
+        self.acquire_timeout = acquire_timeout
+        super(TimeoutPIDLockFile, self).__init__(path, *args, **kwargs)
+
+    def acquire(self, timeout=None, *args, **kwargs):
+        """ Acquire the lock. """
+        if timeout is None:
+            timeout = self.acquire_timeout
+        super(TimeoutPIDLockFile, self).acquire(timeout, *args, **kwargs)
+
 
 def read_pid_from_pidfile(pidfile_path):
     """ Read the PID recorded in the named PID file.
